searchData={"items":[{"type":"module","title":"Ming.CommandProcessor","doc":"Command Process","ref":"Ming.CommandProcessor.html"},{"type":"module","title":"Ming.CompositeRouter","doc":"Composite router allows you to combine multiple router modules into a single\nrouter able to dispatch any registered command from an included child router.\n\nOne example usage is to define a router per context and then combine each\ncontext's router into a single top-level composite app router used for all\ncommand dispatching.","ref":"Ming.CompositeRouter.html"},{"type":"module","title":"Example - Ming.CompositeRouter","doc":"Define a composite router module which imports the commands from each included\n\nrouter:\n\n\n    defmodule Bank.Router do\n      use Ming.CompositeRouter\n\n      router(Bank.Accounts.Router)\n      router(Bank.MoneyTransfer.Router)\n    end\n\n  One or more routers or composite routers can be included in a\n  `Ming.CommandProcessor` since it is also a composite router:\n\n\n    defmodule BankCommandProcessor do\n      use Ming.CommandProcessor\n\n\n      router(Bank.Router)\n    end\n\n\nYou can dispatch a command via the application which will then be routed to\nthe associated child router:\n\n    command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankApp.send(command)\n\nOr via the composite router itself, specifying the application:\n\n    :ok = Bank.AppRouter.publish(command, application: BankApp)\n\n\nA composite router can include composite routers.","ref":"Ming.CompositeRouter.html#module-example"},{"type":"macro","title":"Ming.CompositeRouter.router/1","doc":"Register a `Commanded.Commands.Router` module within this composite router.\n\n\nWill allow the composite router to dispatch any commands registered by the\n\nincluded router module. Multiple routers can be registered.","ref":"Ming.CompositeRouter.html#router/1"},{"type":"module","title":"Ming.ExecutionContext","doc":"Defines the arguments used to execute a command for an aggregate.\n\nThe available options are:\n\n  - `request` - the command to execute, typically a struct\n    (e.g. `%OpenBankAccount{...}`).\n\n  - `retry_attempts` - the number of retries permitted if an\n    `{:error, :wrong_expected_version}` is encountered when appending events.\n\n  - `correlation_id` - a UUID used to correlate related commands/events.\n\n  - `metadata` - a map of key/value pairs containing the metadata to be\n    associated with all events created by the command.\n\n  - `handler` - the module that handles the command. It may be either the\n    aggregate module itself or a separate command handler module.\n\n  - `function` - the name of the function, as an atom, that handles the command.\n    The default value is `:execute`, used to support command dispatch directly\n    to the aggregate module. For command handlers the `:handle` function is\n    used.\n\n  - `before_execute` - the name of the function, as an atom, that prepares the\n    command before execution, called just before `function`. The default value\n    is `nil`, disabling it. It should return `:ok` on success or `{:error, any()}`\n    to cancel the dispatch.","ref":"Ming.ExecutionContext.html"},{"type":"function","title":"Ming.ExecutionContext.format_reply/2","doc":"","ref":"Ming.ExecutionContext.html#format_reply/2"},{"type":"function","title":"Ming.ExecutionContext.retry/1","doc":"","ref":"Ming.ExecutionContext.html#retry/1"},{"type":"module","title":"Ming.ExecutionResult","doc":"Contains the aggregate, events, and metadata created by a successfully\nexecuted command.\n\nThe available fields are:\n\n  - `events` - a list of the created events, it may be an empty list.\n\n  - `metadata` - an map containing the metadata associated with the command\n    dispatch.","ref":"Ming.ExecutionResult.html"},{"type":"type","title":"Ming.ExecutionResult.t/0","doc":"","ref":"Ming.ExecutionResult.html#t:t/0"},{"type":"module","title":"Ming.Handler","doc":"Handler is a process, it allows execution of commands/event.","ref":"Ming.Handler.html"},{"type":"module","title":"Telemetry - Ming.Handler","doc":"* `[:ming, :handler, :execute, :start]`\n  * Description: Emitted when an handler starts executing a request(command/event)\n  * Measurements: `%{system_time: integer()}`\n  * Metadata: `%{application: Ming.CommandProcessor.t(),\n  caller: pid(),\n  execution_context: Ming.ExecutionContext.t()}\n`\n\n* `[:ming, :handler, :execute, :stop]`\n  * Description: Emitted when an handler stops executing a request(command/event)\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Ming.CommandProcessor.t(),\n  caller: pid(),\n  execution_context: Ming.ExecutionContext.t(),\n  response: [map()],\n  error: nil | any()}\n`\n\n* `[:ming, :handler, :execute, :exception]`\n  * Description: Emitted when an handler raises an exception\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Ming.CommandProcessor.t(),\n  caller: pid(),\n  execution_context: Ming.ExecutionContext.t(),\n  kind: :throw | :error | :exit,\n  reason: any(),\n  stacktrace: list()}\n`","ref":"Ming.Handler.html#module-telemetry"},{"type":"function","title":"Ming.Handler.execute/3","doc":"Execute the given command against the aggregate.\n\n  - `context` - includes command execution arguments\n\n    (see `Commanded.Aggregates.ExecutionContext` for details).\n  - `timeout` - an non-negative integer which specifies how many milliseconds\n    to wait for a reply, or the atom :infinity to wait indefinitely.\n    The default value is five seconds (5,000ms).","ref":"Ming.Handler.html#execute/3"},{"type":"function","title":"Return values - Ming.Handler.execute/3","doc":"Returns `{:ok, response}` on success, or `{:error, error}`\non failure.\n\n  - `response` - response produced by the handler, can be an empty list.","ref":"Ming.Handler.html#execute/3-return-values"},{"type":"behaviour","title":"Ming.Middleware","doc":"The middleware behaviour","ref":"Ming.Middleware.html"},{"type":"callback","title":"Ming.Middleware.after_failure/1","doc":"The action executes after failure","ref":"Ming.Middleware.html#c:after_failure/1"},{"type":"callback","title":"Ming.Middleware.before_after/1","doc":"The action executes after dispatch","ref":"Ming.Middleware.html#c:before_after/1"},{"type":"callback","title":"Ming.Middleware.before_dispatch/1","doc":"The action executes before dispatch","ref":"Ming.Middleware.html#c:before_dispatch/1"},{"type":"module","title":"Ming.Pipeline","doc":"Pipeline is a struct used as an argument in the callback functions of modules\nimplementing the `Ming.Middleware` behaviour.\n\nThis struct must be returned by each function to be used in the next\nmiddleware based on the configured middleware chain.","ref":"Ming.Pipeline.html"},{"type":"module","title":"Pipeline fields - Ming.Pipeline","doc":"- `application` - the Ming application.\n  \n  - `assigns` - shared user data as a map.\n\n  - `correlation_id` - an optional UUID used to correlate related\n     commands/events together.\n\n  - `request` - command struct being dispatched.\n\n  - `request_uuid` - UUID assigned to the command being dispatched.\n\n  - `consistency` - requested dispatch consistency, either: `:eventual`\n     (default) or `:strong`.\n\n  - `halted` - flag indicating whether the pipeline was halted.\n\n  - `metadata` - the metadata map to be persisted along with the events.\n\n  - `response` - sets the response to send back to the caller.","ref":"Ming.Pipeline.html#module-pipeline-fields"},{"type":"function","title":"Ming.Pipeline.assign/3","doc":"Puts the `key` with value equal to `value` into `assigns` map.","ref":"Ming.Pipeline.html#assign/3"},{"type":"function","title":"Ming.Pipeline.assign_metadata/3","doc":"Puts the `key` with value equal to `value` into `metadata` map.\n\nNote: Use of atom keys in metadata is deprecated in favour of binary strings.","ref":"Ming.Pipeline.html#assign_metadata/3"},{"type":"function","title":"Ming.Pipeline.chain/3","doc":"Executes the middleware chain.","ref":"Ming.Pipeline.html#chain/3"},{"type":"function","title":"Ming.Pipeline.halt/1","doc":"Halts the pipeline by preventing further middleware downstream from being invoked.\n\n\nPrevents dispatch of the command if `halt` occurs in a `before_dispatch` callback.","ref":"Ming.Pipeline.html#halt/1"},{"type":"function","title":"Ming.Pipeline.halted?/1","doc":"Has the pipeline been halted?","ref":"Ming.Pipeline.html#halted?/1"},{"type":"function","title":"Ming.Pipeline.respond/2","doc":"Sets the response to be returned to the dispatch caller, unless already set.","ref":"Ming.Pipeline.html#respond/2"},{"type":"function","title":"Ming.Pipeline.response/1","doc":"Extract the response from the pipeline","ref":"Ming.Pipeline.html#response/1"},{"type":"type","title":"Ming.Pipeline.t/0","doc":"","ref":"Ming.Pipeline.html#t:t/0"},{"type":"behaviour","title":"Ming.Router","doc":"Command routing macro to allow configuration of each command to its command handler.","ref":"Ming.Router.html"},{"type":"behaviour","title":"Example - Ming.Router","doc":"Define a router module which uses `Ming.Router` and configures\navailable request to dispatch:\n\n\n    defmodule BankRouter do\n      use Ming.Router\n\n\n      dispatch OpenAccount,\n        to: OpenAccountHandler,\n    end\n\nThe `to` option determines which module receives the request being dispatched.\n\nThis command handler module must implement a `handle/1` function. It receives\nthe command to execute.\n\nOnce configured, you can either dispatch a request by using the module and\n\nspecifying the application:\n\n    command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankRouter.send(command, application: BankApp)\n\n\nOr, more simply, you should include the router module in your application:\n\n    defmodule CommandProcssor do\n      use Ming.CommandProcssor , otp_app: :my_app\n\n      router MyApp.Router\n    end\n\nThen dispatch commands using the app:\n\n    command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankApp.send(command)\n\nAlternatively, you may specify the name of a function on your handle module to which the command will be dispatched:","ref":"Ming.Router.html#module-example"},{"type":"behaviour","title":"Example - Ming.Router","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n\n      # Will route to `BankAccount.open_account/2`\n      dispatch OpenAccount, to: BankAccount, function: :open_account \n    end","ref":"Ming.Router.html#module-example"},{"type":"behaviour","title":"Metadata - Ming.Router","doc":"You can associate metadata with all events created by the command.\n\nSupply a map containing key/value pairs comprising the metadata:\n\n    :ok = BankApp.send(command, metadata: %{\"ip_address\" => \"127.0.0.1\"})","ref":"Ming.Router.html#module-metadata"},{"type":"macro","title":"Ming.Router.dispatch/2","doc":"Configure the command, or list of commands, to be dispatched to the\ncorresponding handler.","ref":"Ming.Router.html#dispatch/2"},{"type":"macro","title":"Example - Ming.Router.dispatch/2","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount\n    end","ref":"Ming.Router.html#dispatch/2-example"},{"type":"macro","title":"Ming.Router.middleware/1","doc":"Include the given middleware module to be called before and after\nsuccess or failure of each command dispatch(send/publish/post)\n\nThe middleware module must implement the `Ming.Middleware` behaviour.\n\nMiddleware modules are executed in the order they are defined.","ref":"Ming.Router.html#middleware/1"},{"type":"macro","title":"Example - Ming.Router.middleware/1","doc":"defmodule BankRouter do\n      use Ming.Router\n\n      middleware CommandLogger\n      middleware MyCommandValidator\n      middleware AuthorizeCommand\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount\n    end","ref":"Ming.Router.html#middleware/1-example"},{"type":"macro","title":"Ming.Router.post_middleware/1","doc":"Include the given middleware module to be called before and after\nsuccess or failure of each message dispatch using post \n\nThe middleware module must implement the `Ming.Middleware` behaviour.\n\nMiddleware modules are executed in the order they are defined.","ref":"Ming.Router.html#post_middleware/1"},{"type":"macro","title":"Example - Ming.Router.post_middleware/1","doc":"defmodule BankRouter do\n      use Ming.Router\n\n      post_middleware CommandLogger\n      post_middleware MyCommandValidator\n      post_middleware AuthorizeCommand\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount\n    end","ref":"Ming.Router.html#post_middleware/1-example"},{"type":"callback","title":"Ming.Router.publish/1","doc":"Dispatch the given event to all registered handler.\n\nReturns `:ok` or `{:ok, any()}` on success, or `{:error, reason}` on failure.","ref":"Ming.Router.html#c:publish/1"},{"type":"callback","title":"Example - Ming.Router.publish/1","doc":"event = %AccountOpened{account_number: \"ACC123\", initial_balance: 1_000}\n    :ok = BankRouter.publish(event)","ref":"Ming.Router.html#c:publish/1-example"},{"type":"callback","title":"Ming.Router.publish/2","doc":"Dispatch the given command to all registered handler.\n\n  - `command` is a command struct which must be registered with the router.\n\n  - `timeout_or_opts` is either an integer timeout, `:infinity`, or a keyword\n    list of options.\n\n    The timeout must be an integer greater than zero which specifies how many\n    milliseconds to allow the command to be handled, or the atom `:infinity`\n    to wait indefinitely. The default timeout value is five seconds.\n\n\n    Alternatively, an options keyword list can be provided with the following\n    options.\n\n    Options:\n\n      - `request_uuid` - an optional UUID used to identify the command being\n        dispatched.\n\n      - `correlation_id` - an optional UUID used to correlate related\n        commands/events together.\n\n      - `metadata` - an optional map containing key/value pairs comprising\n        the metadata to be associated with all events created by the\n        command.\n\n     - `timeout` - as described above.\n\n     - `concurrency_timeout` - the maximum amount of time (in milliseconds or :infinity) each task is allowed to execute for. Defaults to 30_000 \n\n     - `max_concurrency` - sets the maximum number of tasks to run at the same time. Defaults to`System.schedulers_online/0`.\n\nReturns `:ok` on success, or `{:error, reason}` on failure.","ref":"Ming.Router.html#c:publish/2"},{"type":"callback","title":"Example - Ming.Router.publish/2","doc":"command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankRouter.send(command, consistency: :strong, timeout: 30_000)","ref":"Ming.Router.html#c:publish/2-example"},{"type":"macro","title":"Ming.Router.publish_middleware/1","doc":"Include the given middleware module to be called before and after\nsuccess or failure of each message dispatch using publish\n\nThe middleware module must implement the `Ming.Middleware` behaviour.\n\nMiddleware modules are executed in the order they are defined.","ref":"Ming.Router.html#publish_middleware/1"},{"type":"macro","title":"Example - Ming.Router.publish_middleware/1","doc":"defmodule BankRouter do\n      use Ming.Router\n\n      publish_middleware CommandLogger\n      publish_middleware MyCommandValidator\n      publish_middleware AuthorizeCommand\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount\n    end","ref":"Ming.Router.html#publish_middleware/1-example"},{"type":"type","title":"Ming.Router.publish_resp/0","doc":"","ref":"Ming.Router.html#t:publish_resp/0"},{"type":"callback","title":"Ming.Router.send/1","doc":"Dispatch the given command to one registered handler.\n\nReturns `:ok` on success, or `{:error, reason}` on failure.","ref":"Ming.Router.html#c:send/1"},{"type":"callback","title":"Example - Ming.Router.send/1","doc":"command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n    :ok = BankRouter.send(command)","ref":"Ming.Router.html#c:send/1-example"},{"type":"callback","title":"Ming.Router.send/2","doc":"Dispatch the given command to one registered handler.\n\n  - `command` is a command struct which must be registered with the router.\n\n  - `timeout_or_opts` is either an integer timeout, `:infinity`, or a keyword\n    list of options.\n\n    The timeout must be an integer greater than zero which specifies how many\n    milliseconds to allow the command to be handled, or the atom `:infinity`\n    to wait indefinitely. The default timeout value is five seconds.\n\n\n    Alternatively, an options keyword list can be provided with the following\n    options.\n\n    Options:\n\n      - `request_uuid` - an optional UUID used to identify the command being\n        dispatched.\n\n      - `correlation_id` - an optional UUID used to correlate related\n        commands/events together.\n\n      - `metadata` - an optional map containing key/value pairs comprising\n        the metadata to be associated with all events created by the\n        command.\n\n     - `timeout` - as described above.\n\nReturns `:ok` on success, or `{:error, reason}` on failure.","ref":"Ming.Router.html#c:send/2"},{"type":"callback","title":"Example - Ming.Router.send/2","doc":"command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankRouter.send(command, consistency: :strong, timeout: 30_000)","ref":"Ming.Router.html#c:send/2-example"},{"type":"macro","title":"Ming.Router.send_middleware/1","doc":"Include the given middleware module to be called before and after\nsuccess or failure of each message dispatch using send\n\nThe middleware module must implement the `Ming.Middleware` behaviour.\n\nMiddleware modules are executed in the order they are defined.","ref":"Ming.Router.html#send_middleware/1"},{"type":"macro","title":"Example - Ming.Router.send_middleware/1","doc":"defmodule BankRouter do\n      use Ming.Router\n\n      send_middleware CommandLogger\n      send_middleware MyCommandValidator\n      send_middleware AuthorizeCommand\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount\n    end","ref":"Ming.Router.html#send_middleware/1-example"},{"type":"type","title":"Ming.Router.send_resp/0","doc":"","ref":"Ming.Router.html#t:send_resp/0"},{"type":"module","title":"Ming.UUID","doc":"Generate and format random, version 4 UUIDs.","ref":"Ming.UUID.html"},{"type":"function","title":"Ming.UUID.binary_to_string!/1","doc":"","ref":"Ming.UUID.html#binary_to_string!/1"},{"type":"type","title":"Ming.UUID.hex/0","doc":"A hex-encoded UUID string.","ref":"Ming.UUID.html#t:hex/0"},{"type":"type","title":"Ming.UUID.raw/0","doc":"A raw binary representation of a UUID.","ref":"Ming.UUID.html#t:raw/0"},{"type":"function","title":"Ming.UUID.string_to_binary!/1","doc":"Converts a string representing a UUID into a raw binary.","ref":"Ming.UUID.html#string_to_binary!/1"},{"type":"function","title":"Ming.UUID.uuid4/0","doc":"Generates a random, version 4 UUID.","ref":"Ming.UUID.html#uuid4/0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}